using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Runtime.Serialization;
using System.Runtime.ConstrainedExecution;

namespace BiDirHash
{
    /// <summary>
    /// It extends the Hashtable and so it has all the interface by default viz. 
    /// IDictionary, ICollection, IEnumerable, ISerializable, IDeserializationCallback, ICloneable
    /// 
    /// The biggest disadvantage of such implementation is your value and key both must be unique.
    /// </summary>
    class BiDirHashtable : Hashtable
    {
        #region Constructors
        public BiDirHashtable()
            : base()
        {
        }

        public BiDirHashtable(IDictionary d)
            : base(d)
        {
        }

        public BiDirHashtable(IEqualityComparer equalityComparer)
            : base(equalityComparer)
        { 
        }

        public BiDirHashtable(int capacity)
            : base(capacity)
        { 
        }

        public BiDirHashtable(IDictionary d, float loadFactor)
            : base(d, loadFactor) 
        {
        }

        public BiDirHashtable(IDictionary d, IEqualityComparer equalityComparer)
            : base(d, equalityComparer)
        { 
        }

        [Obsolete("Please use BiDirHashtable(IEqualityComparer) instead.")]
        public BiDirHashtable(IHashCodeProvider hcp, IComparer comparer)
            : base(hcp, comparer)
        { 
        }

        public BiDirHashtable(int capacity, float loadFactor)
            : base(capacity, loadFactor)
        {
        }

        public BiDirHashtable(int capacity, IEqualityComparer equalityComparer)
            : base(capacity, equalityComparer)
        {
        }

        protected BiDirHashtable(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }

        public BiDirHashtable(IDictionary d, float loadFactor, IEqualityComparer equalityComparer)
            : base(d, loadFactor, equalityComparer)
        {
        }

        [Obsolete("Please use BiDirHashtable(IDictionary, IEqualityComparer) instead.")]
        public BiDirHashtable(IDictionary d, IHashCodeProvider hcp, IComparer comparer)
            : base(d, hcp, comparer)
        {
        }

        public BiDirHashtable(int capacity, float loadFactor, IEqualityComparer equalityComparer)
            : base(capacity, loadFactor, equalityComparer)
        { 
        }

        [Obsolete("Please use BiDirHashtable(int, IEqualityComparer) instead.")]
        public BiDirHashtable(int capacity, IHashCodeProvider hcp, IComparer comparer)
            : base(capacity, hcp, comparer)
        {
        }

        [Obsolete("Please use BiDirHashtable(IDictionary, float, IEqualityComparer) instead.")]
        public BiDirHashtable(IDictionary d, float loadFactor, IHashCodeProvider hcp, IComparer comparer)
            : base(d, loadFactor, hcp, comparer)
        { 
        }

        [Obsolete("Please use BiDirHashtable(int, float, IEqualityComparer) instead.")]
        public BiDirHashtable(int capacity, float loadFactor, IHashCodeProvider hcp, IComparer comparer)
            : base(capacity, loadFactor, hcp, comparer)
        {
        }
        #endregion

        #region Overridable
        /// <summary>
        /// Adds the new key / value pair to the hashtable. For bi-directional 
        /// hashing it adds the key -> value as well as value -> key combination
        /// </summary>
        /// <param name="key">Key to be added as key as well as value</param>
        /// <param name="value">Value to be added as key as well as value</param>
        public override void Add(object key, object value) 
        {
            base.Add(key, value);
            base.Add(value, key);
        }

        /// <summary>
        /// Removes the key -> value as well as value -> key pair. The order of 
        /// removing is important. You should not be removing the key before the 
        /// value as you will not be able to get the other object you want to remove 
        /// </summary>
        /// <param name="key"></param>
        public override void Remove(object key)
        {
            base.Remove(base[key]);
            base.Remove(key);
        }

        /// <summary>
        /// As we are adding double elements we will retrun the count of the hashtable halfed
        /// </summary>
        public override int Count 
        {
            get
            {
                return base.Count / 2;
            }
        }
        #endregion
    }
}
